@@ -1,4 +1,4 @@
-/*	$NetBSD: common.c,v 1.29 2012/03/24 20:08:43 christos Exp $	*/
+/*	$NetBSD: common.c,v 1.24 2009/12/30 22:37:40 christos Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -37,7 +37,7 @@
 #if 0
 static char sccsid[] = "@(#)common.c	8.1 (Berkeley) 6/4/93";
 #else
-__RCSID("$NetBSD: common.c,v 1.29 2012/03/24 20:08:43 christos Exp $");
+__RCSID("$NetBSD: common.c,v 1.24 2009/12/30 22:37:40 christos Exp $");
 #endif
 #endif /* not lint && not SCCSID */
 
@@ -57,7 +57,7 @@ ed_end_of_file(EditLine *el, Int c __attribute__((__unused__)))
 
 	re_goto_bottom(el);
 	*el->el_line.lastchar = '\0';
-	return CC_EOF;
+	return (CC_EOF);
 }
 
 
@@ -71,7 +71,7 @@ ed_insert(EditLine *el, Int c)
 	int count = el->el_state.argument;
 
 	if (c == '\0')
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	if (el->el_line.lastchar + el->el_state.argument >=
 	    el->el_line.limit) {
@@ -99,7 +99,7 @@ ed_insert(EditLine *el, Int c)
 	if (el->el_state.inputmode == MODE_REPLACE_1)
 		return vi_command_mode(el, 0);
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -114,7 +114,7 @@ ed_delete_prev_word(EditLine *el, Int c __attribute__((__unused__)))
 	Char *cp, *p, *kp;
 
 	if (el->el_line.cursor == el->el_line.buffer)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	cp = c__prev_word(el->el_line.cursor, el->el_line.buffer,
 	    el->el_state.argument, ce__isword);
@@ -127,7 +127,7 @@ ed_delete_prev_word(EditLine *el, Int c __attribute__((__unused__)))
 	el->el_line.cursor = cp;
 	if (el->el_line.cursor < el->el_line.buffer)
 		el->el_line.cursor = el->el_line.buffer; /* bounds check */
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -137,9 +137,9 @@ ed_delete_prev_word(EditLine *el, Int c __attribute__((__unused__)))
  */
 protected el_action_t
 /*ARGSUSED*/
-ed_delete_next_char(EditLine *el, Int c __attribute__((__unused__)))
+ed_delete_next_char(EditLine *el, Int c)
 {
-#ifdef DEBUG_EDIT
+#ifdef notdef			/* XXX */
 #define	EL	el->el_line
 	(void) fprintf(el->el_errlfile,
 	    "\nD(b: %x(%s)  c: %x(%s) last: %x(%s) limit: %x(%s)\n",
@@ -152,29 +152,32 @@ ed_delete_next_char(EditLine *el, Int c __attribute__((__unused__)))
 			if (el->el_line.cursor == el->el_line.buffer) {
 				/* if I'm also at the beginning */
 #ifdef KSHVI
-				return CC_ERROR;
+				return (CC_ERROR);
 #else
 				/* then do an EOF */
-				terminal_writec(el, c);
-				return CC_EOF;
+				term_writec(el, c);
+				return (CC_EOF);
 #endif
 			} else {
 #ifdef KSHVI
 				el->el_line.cursor--;
 #else
-				return CC_ERROR;
+				return (CC_ERROR);
 #endif
 			}
-		} else
-				return CC_ERROR;
+		} else {
+			if (el->el_line.cursor != el->el_line.buffer)
+				el->el_line.cursor--;
+			else
+				return (CC_ERROR);
+		}
 	}
 	c_delafter(el, el->el_state.argument);	/* delete after dot */
-	if (el->el_map.type == MAP_VI &&
-	    el->el_line.cursor >= el->el_line.lastchar &&
+	if (el->el_line.cursor >= el->el_line.lastchar &&
 	    el->el_line.cursor > el->el_line.buffer)
 			/* bounds check */
 		el->el_line.cursor = el->el_line.lastchar - 1;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -195,7 +198,7 @@ ed_kill_line(EditLine *el, Int c __attribute__((__unused__)))
 	el->el_chared.c_kill.last = kp;
 			/* zap! -- delete to end */
 	el->el_line.lastchar = el->el_line.cursor;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -212,13 +215,13 @@ ed_move_to_end(EditLine *el, Int c __attribute__((__unused__)))
 	if (el->el_map.type == MAP_VI) {
 		if (el->el_chared.c_vcmd.action != NOP) {
 			cv_delfini(el);
-			return CC_REFRESH;
+			return (CC_REFRESH);
 		}
 #ifdef VI_MOVE
 		el->el_line.cursor--;
 #endif
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -239,10 +242,10 @@ ed_move_to_beg(EditLine *el, Int c __attribute__((__unused__)))
 			el->el_line.cursor++;
 		if (el->el_chared.c_vcmd.action != NOP) {
 			cv_delfini(el);
-			return CC_REFRESH;
+			return (CC_REFRESH);
 		}
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -256,7 +259,7 @@ ed_transpose_chars(EditLine *el, Int c)
 
 	if (el->el_line.cursor < el->el_line.lastchar) {
 		if (el->el_line.lastchar <= &el->el_line.buffer[1])
-			return CC_ERROR;
+			return (CC_ERROR);
 		else
 			el->el_line.cursor++;
 	}
@@ -265,9 +268,9 @@ ed_transpose_chars(EditLine *el, Int c)
 		c = el->el_line.cursor[-2];
 		el->el_line.cursor[-2] = el->el_line.cursor[-1];
 		el->el_line.cursor[-1] = c;
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	} else
-		return CC_ERROR;
+		return (CC_ERROR);
 }
 
 
@@ -285,7 +288,7 @@ ed_next_char(EditLine *el, Int c __attribute__((__unused__)))
 	    (el->el_line.cursor == lim - 1 &&
 	    el->el_map.type == MAP_VI &&
 	    el->el_chared.c_vcmd.action == NOP))
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor += el->el_state.argument;
 	if (el->el_line.cursor > lim)
@@ -294,9 +297,9 @@ ed_next_char(EditLine *el, Int c __attribute__((__unused__)))
 	if (el->el_map.type == MAP_VI)
 		if (el->el_chared.c_vcmd.action != NOP) {
 			cv_delfini(el);
-			return CC_REFRESH;
+			return (CC_REFRESH);
 		}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -310,7 +313,7 @@ ed_prev_word(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor == el->el_line.buffer)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = c__prev_word(el->el_line.cursor,
 	    el->el_line.buffer,
@@ -320,9 +323,9 @@ ed_prev_word(EditLine *el, Int c __attribute__((__unused__)))
 	if (el->el_map.type == MAP_VI)
 		if (el->el_chared.c_vcmd.action != NOP) {
 			cv_delfini(el);
-			return CC_REFRESH;
+			return (CC_REFRESH);
 		}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -343,11 +346,11 @@ ed_prev_char(EditLine *el, Int c __attribute__((__unused__)))
 		if (el->el_map.type == MAP_VI)
 			if (el->el_chared.c_vcmd.action != NOP) {
 				cv_delfini(el);
-				return CC_REFRESH;
+				return (CC_REFRESH);
 			}
-		return CC_CURSOR;
+		return (CC_CURSOR);
 	} else
-		return CC_ERROR;
+		return (CC_ERROR);
 }
 
 
@@ -366,9 +369,9 @@ ed_quoted_insert(EditLine *el, Int c)
 	c = tc;
 	tty_noquotemode(el);
 	if (num == 1)
-		return ed_insert(el, c);
+		return (ed_insert(el, c));
 	else
-		return ed_end_of_file(el, 0);
+		return (ed_end_of_file(el, 0));
 }
 
 
@@ -380,7 +383,7 @@ ed_digit(EditLine *el, Int c)
 {
 
 	if (!Isdigit(c))
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	if (el->el_state.doingarg) {
 			/* if doing an arg, add this in... */
@@ -388,11 +391,11 @@ ed_digit(EditLine *el, Int c)
 			el->el_state.argument = c - '0';
 		else {
 			if (el->el_state.argument > 1000000)
-				return CC_ERROR;
+				return (CC_ERROR);
 			el->el_state.argument =
 			    (el->el_state.argument * 10) + (c - '0');
 		}
-		return CC_ARGHACK;
+		return (CC_ARGHACK);
 	}
 
 	return ed_insert(el, c);
@@ -408,18 +411,18 @@ ed_argument_digit(EditLine *el, Int c)
 {
 
 	if (!Isdigit(c))
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	if (el->el_state.doingarg) {
 		if (el->el_state.argument > 1000000)
-			return CC_ERROR;
+			return (CC_ERROR);
 		el->el_state.argument = (el->el_state.argument * 10) +
 		    (c - '0');
 	} else {		/* else starting an argument */
 		el->el_state.argument = c - '0';
 		el->el_state.doingarg = 1;
 	}
-	return CC_ARGHACK;
+	return (CC_ARGHACK);
 }
 
 
@@ -429,11 +432,10 @@ ed_argument_digit(EditLine *el, Int c)
  */
 protected el_action_t
 /*ARGSUSED*/
-ed_unassigned(EditLine *el __attribute__((__unused__)),
-    Int c __attribute__((__unused__)))
+ed_unassigned(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	return CC_ERROR;
+	return (CC_ERROR);
 }
 
 
@@ -451,7 +453,7 @@ ed_tty_sigint(EditLine *el __attribute__((__unused__)),
 	      Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -465,7 +467,7 @@ ed_tty_dsusp(EditLine *el __attribute__((__unused__)),
 	     Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -479,7 +481,7 @@ ed_tty_flush_output(EditLine *el __attribute__((__unused__)),
 		    Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -493,7 +495,7 @@ ed_tty_sigquit(EditLine *el __attribute__((__unused__)),
 	       Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -507,7 +509,7 @@ ed_tty_sigtstp(EditLine *el __attribute__((__unused__)),
 	       Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -521,7 +523,7 @@ ed_tty_stop_output(EditLine *el __attribute__((__unused__)),
 		   Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -535,7 +537,7 @@ ed_tty_start_output(EditLine *el __attribute__((__unused__)),
 		    Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -551,7 +553,7 @@ ed_newline(EditLine *el, Int c __attribute__((__unused__)))
 	re_goto_bottom(el);
 	*el->el_line.lastchar++ = '\n';
 	*el->el_line.lastchar = '\0';
-	return CC_NEWLINE;
+	return (CC_NEWLINE);
 }
 
 
@@ -565,13 +567,13 @@ ed_delete_prev_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor <= el->el_line.buffer)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	c_delbefore(el, el->el_state.argument);
 	el->el_line.cursor -= el->el_state.argument;
 	if (el->el_line.cursor < el->el_line.buffer)
 		el->el_line.cursor = el->el_line.buffer;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -584,9 +586,9 @@ protected el_action_t
 ed_clear_screen(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	terminal_clear_screen(el);	/* clear the whole real screen */
+	term_clear_screen(el);	/* clear the whole real screen */
 	re_clear_display(el);	/* reset everything */
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -600,7 +602,7 @@ ed_redisplay(EditLine *el __attribute__((__unused__)),
 	     Int c __attribute__((__unused__)))
 {
 
-	return CC_REDISPLAY;
+	return (CC_REDISPLAY);
 }
 
 
@@ -614,7 +616,7 @@ ed_start_over(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	ch_reset(el, 0);
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -628,7 +630,7 @@ ed_sequence_lead_in(EditLine *el __attribute__((__unused__)),
 		    Int c __attribute__((__unused__)))
 {
 
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -718,7 +720,7 @@ ed_search_prev_history(EditLine *el, Int c __attribute__((__unused__)))
 		    "e_prev_search_hist(): eventno < 0;\n");
 #endif
 		el->el_history.eventno = 0;
-		return CC_ERROR;
+		return (CC_ERROR);
 	}
 	if (el->el_history.eventno == 0) {
 		(void) Strncpy(el->el_history.buf, el->el_line.buffer,
@@ -727,11 +729,11 @@ ed_search_prev_history(EditLine *el, Int c __attribute__((__unused__)))
 		    (el->el_line.lastchar - el->el_line.buffer);
 	}
 	if (el->el_history.ref == NULL)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	hp = HIST_FIRST(el);
 	if (hp == NULL)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	c_setpat(el);		/* Set search pattern !! */
 
@@ -757,11 +759,11 @@ ed_search_prev_history(EditLine *el, Int c __attribute__((__unused__)))
 #ifdef SDEBUG
 		(void) fprintf(el->el_errfile, "not found\n");
 #endif
-		return CC_ERROR;
+		return (CC_ERROR);
 	}
 	el->el_history.eventno = h;
 
-	return hist_get(el);
+	return (hist_get(el));
 }
 
 
@@ -782,14 +784,14 @@ ed_search_next_history(EditLine *el, Int c __attribute__((__unused__)))
 	*el->el_line.lastchar = '\0';	/* just in case */
 
 	if (el->el_history.eventno == 0)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	if (el->el_history.ref == NULL)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	hp = HIST_FIRST(el);
 	if (hp == NULL)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	c_setpat(el);		/* Set search pattern !! */
 
@@ -810,12 +812,12 @@ ed_search_next_history(EditLine *el, Int c __attribute__((__unused__)))
 #ifdef SDEBUG
 			(void) fprintf(el->el_errfile, "not found\n");
 #endif
-			return CC_ERROR;
+			return (CC_ERROR);
 		}
 	}
 	el->el_history.eventno = found;
 
-	return hist_get(el);
+	return (hist_get(el));
 }
 
 
@@ -841,7 +843,7 @@ ed_prev_line(EditLine *el, Int c __attribute__((__unused__)))
 			break;
 
 	if (el->el_state.argument > 0)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	/*
          * Move to the beginning of the line
@@ -858,7 +860,7 @@ ed_prev_line(EditLine *el, Int c __attribute__((__unused__)))
 		continue;
 
 	el->el_line.cursor = ptr;
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -881,7 +883,7 @@ ed_next_line(EditLine *el, Int c __attribute__((__unused__)))
 			break;
 
 	if (el->el_state.argument > 0)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	/*
          * Move to the character requested
@@ -892,7 +894,7 @@ ed_next_line(EditLine *el, Int c __attribute__((__unused__)))
 		continue;
 
 	el->el_line.cursor = ptr;
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -908,10 +910,10 @@ ed_command(EditLine *el, Int c __attribute__((__unused__)))
 	int tmplen;
 
 	tmplen = c_gets(el, tmpbuf, STR("\n: "));
-	terminal__putc(el, '\n');
+	term__putc(el, '\n');
 
 	if (tmplen < 0 || (tmpbuf[tmplen] = 0, parse_line(el, tmpbuf)) == -1)
-		terminal_beep(el);
+		term_beep(el);
 
 	el->el_map.current = el->el_map.key;
 	re_clear_display(el);
