@@ -1,4 +1,4 @@
-/*	$NetBSD: unvis.c,v 1.44 2014/09/26 15:43:36 roy Exp $	*/
+/*	$NetBSD: unvis.c,v 1.36 2011/03/18 09:07:20 martin Exp $	*/
 
 /*-
  * Copyright (c) 1989, 1993
@@ -34,17 +34,16 @@
 #if 0
 static char sccsid[] = "@(#)unvis.c	8.1 (Berkeley) 6/4/93";
 #else
-__RCSID("$NetBSD: unvis.c,v 1.44 2014/09/26 15:43:36 roy Exp $");
+__RCSID("$NetBSD: unvis.c,v 1.36 2011/03/18 09:07:20 martin Exp $");
 #endif
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/types.h>
+#include <stdint.h>
 
 #include <assert.h>
 #include <ctype.h>
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#endif
 #include <stdio.h>
 #include <errno.h>
 #include <vis.h>
@@ -64,25 +63,24 @@ __weak_alias(strnunvisx,_strnunvisx)
 #define	S_CTRL		4	/* control char started (^) */
 #define	S_OCTAL2	5	/* octal digit 2 */
 #define	S_OCTAL3	6	/* octal digit 3 */
-#define	S_HEX		7	/* mandatory hex digit */
-#define	S_HEX1		8	/* http hex digit */
-#define	S_HEX2		9	/* http hex digit 2 */
-#define	S_MIME1		10	/* mime hex digit 1 */
-#define	S_MIME2		11	/* mime hex digit 2 */
-#define	S_EATCRNL	12	/* mime eating CRNL */
-#define	S_AMP		13	/* seen & */
-#define	S_NUMBER	14	/* collecting number */
-#define	S_STRING	15	/* collecting string */
+#define	S_HEX1		7	/* http hex digit */
+#define	S_HEX2		8	/* http hex digit 2 */
+#define S_MIME1		9	/* mime hex digit 1 */
+#define S_MIME2		10	/* mime hex digit 2 */
+#define S_EATCRNL	11	/* mime eating CRNL */
+#define S_AMP		12	/* seen & */
+#define S_NUMBER	13	/* collecting number */
+#define S_STRING	14	/* collecting string */
 
 #define	isoctal(c)	(((u_char)(c)) >= '0' && ((u_char)(c)) <= '7')
-#define	xtod(c)		(isdigit(c) ? (c - '0') : ((tolower(c) - 'a') + 10))
-#define	XTOD(c)		(isdigit(c) ? (c - '0') : ((c - 'A') + 10))
+#define xtod(c)		(isdigit(c) ? (c - '0') : ((tolower(c) - 'a') + 10))
+#define XTOD(c)		(isdigit(c) ? (c - '0') : ((c - 'A') + 10))
 
 /*
  * RFC 1866
  */
 static const struct nv {
-	char name[7];
+	const char *name;
 	uint8_t value;
 } nv[] = {
 	{ "AElig",	198 }, /* capital AE diphthong (ligature)  */
@@ -299,9 +297,6 @@ unvis(char *cp, int c, int *astate, int flag)
 			*cp = '\033';
 			*astate = SS(0, S_GROUND);
 			return UNVIS_VALID;
-		case 'x':
-			*astate = SS(0, S_HEX);
-			return UNVIS_NOCHAR;
 		case '\n':
 			/*
 			 * hidden newline
@@ -314,12 +309,6 @@ unvis(char *cp, int c, int *astate, int flag)
 			 */
 			*astate = SS(0, S_GROUND);
 			return UNVIS_NOCHAR;
-		default:
-			if (isgraph(c)) {
-				*cp = c;
-				*astate = SS(0, S_GROUND);
-				return UNVIS_VALID;
-			}
 		}
 		goto bad;
 
@@ -371,10 +360,6 @@ unvis(char *cp, int c, int *astate, int flag)
 		 */
 		return UNVIS_VALIDPUSH;
 
-	case S_HEX:
-		if (!isxdigit(uc))
-			goto bad;
-		/*FALLTHROUGH*/
 	case S_HEX1:
 		if (isxdigit(uc)) {
 			*cp = xtod(uc);
@@ -495,7 +480,7 @@ int
 strnunvisx(char *dst, size_t dlen, const char *src, int flag)
 {
 	char c;
-	char t = '\0', *start = dst;
+	char t, *start = dst;
 	int state = 0;
 
 	_DIAGASSERT(src != NULL);
@@ -526,7 +511,7 @@ strnunvisx(char *dst, size_t dlen, const char *src, int flag)
 			errno = EINVAL;
 			return -1;
 		default:
-			_DIAGASSERT(/*CONSTCOND*/0);
+			_DIAGASSERT(0);
 			errno = EINVAL;
 			return -1;
 		}
