@@ -1,4 +1,4 @@
-/*	$NetBSD: vi.c,v 1.64 2021/08/28 17:17:47 christos Exp $	*/
+/*	$NetBSD: vi.c,v 1.34 2011/02/22 05:45:08 joerg Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -33,43 +33,37 @@
  */
 
 #include "config.h"
+#include <stdlib.h>
+#include <unistd.h>
+#include <limits.h>
+#include <sys/wait.h>
+
 #if !defined(lint) && !defined(SCCSID)
 #if 0
 static char sccsid[] = "@(#)vi.c	8.1 (Berkeley) 6/4/93";
 #else
-__RCSID("$NetBSD: vi.c,v 1.64 2021/08/28 17:17:47 christos Exp $");
+__RCSID("$NetBSD: vi.c,v 1.34 2011/02/22 05:45:08 joerg Exp $");
 #endif
 #endif /* not lint && not SCCSID */
 
 /*
  * vi.c: Vi mode commands.
  */
-#include <sys/wait.h>
-#include <ctype.h>
-#include <limits.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
 #include "el.h"
-#include "common.h"
-#include "emacs.h"
-#include "fcns.h"
-#include "vi.h"
 
-static el_action_t	cv_action(EditLine *, wint_t);
-static el_action_t	cv_paste(EditLine *, wint_t);
+private el_action_t	cv_action(EditLine *, Int);
+private el_action_t	cv_paste(EditLine *, Int);
 
 /* cv_action():
  *	Handle vi actions.
  */
-static el_action_t
-cv_action(EditLine *el, wint_t c)
+private el_action_t
+cv_action(EditLine *el, Int c)
 {
 
 	if (el->el_chared.c_vcmd.action != NOP) {
 		/* 'cc', 'dd' and (possibly) friends */
-		if (c != (wint_t)el->el_chared.c_vcmd.action)
+		if (c != el->el_chared.c_vcmd.action)
 			return CC_ERROR;
 
 		if (!(c & YANK))
@@ -85,27 +79,26 @@ cv_action(EditLine *el, wint_t c)
 		if (c & INSERT)
 			el->el_map.current = el->el_map.key;
 
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
 	el->el_chared.c_vcmd.pos = el->el_line.cursor;
 	el->el_chared.c_vcmd.action = c;
-	return CC_ARGHACK;
+	return (CC_ARGHACK);
 }
 
 /* cv_paste():
  *	Paste previous deletion before or after the cursor
  */
-static el_action_t
-cv_paste(EditLine *el, wint_t c)
+private el_action_t
+cv_paste(EditLine *el, Int c)
 {
 	c_kill_t *k = &el->el_chared.c_kill;
 	size_t len = (size_t)(k->last - k->buf);
 
 	if (k->buf == NULL || len == 0)
-		return CC_ERROR;
+		return (CC_ERROR);
 #ifdef DEBUG_PASTE
-	(void) fprintf(el->el_errfile, "Paste: \"%.*ls\"\n", (int)len,
-	    k->buf);
+	(void) fprintf(el->el_errfile, "Paste: \"%.*s\"\n", (int)len, k->buf);
 #endif
 
 	cv_undo(el);
@@ -115,11 +108,11 @@ cv_paste(EditLine *el, wint_t c)
 
 	c_insert(el, (int)len);
 	if (el->el_line.cursor + len > el->el_line.lastchar)
-		return CC_ERROR;
+		return (CC_ERROR);
 	(void) memcpy(el->el_line.cursor, k->buf, len *
 	    sizeof(*el->el_line.cursor));
 
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -127,12 +120,12 @@ cv_paste(EditLine *el, wint_t c)
  *	Vi paste previous deletion to the right of the cursor
  *	[p]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_paste_next(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_paste_next(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	return cv_paste(el, 0);
+	return (cv_paste(el, 0));
 }
 
 
@@ -140,12 +133,12 @@ vi_paste_next(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi paste previous deletion to the left of the cursor
  *	[P]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_paste_prev(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_paste_prev(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	return cv_paste(el, 1);
+	return (cv_paste(el, 1));
 }
 
 
@@ -153,13 +146,13 @@ vi_paste_prev(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the previous space delimited word
  *	[B]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_prev_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_prev_big_word(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor == el->el_line.buffer)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = cv_prev_word(el->el_line.cursor,
 	    el->el_line.buffer,
@@ -168,9 +161,9 @@ vi_prev_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
 
 	if (el->el_chared.c_vcmd.action != NOP) {
 		cv_delfini(el);
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -178,13 +171,13 @@ vi_prev_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the previous word
  *	[b]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_prev_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_prev_word(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor == el->el_line.buffer)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = cv_prev_word(el->el_line.cursor,
 	    el->el_line.buffer,
@@ -193,9 +186,9 @@ vi_prev_word(EditLine *el, wint_t c __attribute__((__unused__)))
 
 	if (el->el_chared.c_vcmd.action != NOP) {
 		cv_delfini(el);
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -203,13 +196,13 @@ vi_prev_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the next space delimited word
  *	[W]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_next_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_next_big_word(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor >= el->el_line.lastchar - 1)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = cv_next_word(el, el->el_line.cursor,
 	    el->el_line.lastchar, el->el_state.argument, cv__isWord);
@@ -217,9 +210,9 @@ vi_next_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
 	if (el->el_map.type == MAP_VI)
 		if (el->el_chared.c_vcmd.action != NOP) {
 			cv_delfini(el);
-			return CC_REFRESH;
+			return (CC_REFRESH);
 		}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -227,13 +220,13 @@ vi_next_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the next word
  *	[w]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_next_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_next_word(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor >= el->el_line.lastchar - 1)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = cv_next_word(el, el->el_line.cursor,
 	    el->el_line.lastchar, el->el_state.argument, cv__isword);
@@ -241,9 +234,9 @@ vi_next_word(EditLine *el, wint_t c __attribute__((__unused__)))
 	if (el->el_map.type == MAP_VI)
 		if (el->el_chared.c_vcmd.action != NOP) {
 			cv_delfini(el);
-			return CC_REFRESH;
+			return (CC_REFRESH);
 		}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -251,21 +244,21 @@ vi_next_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi change case of character under the cursor and advance one character
  *	[~]
  */
-libedit_private el_action_t
-vi_change_case(EditLine *el, wint_t c)
+protected el_action_t
+vi_change_case(EditLine *el, Int c)
 {
 	int i;
 
 	if (el->el_line.cursor >= el->el_line.lastchar)
-		return CC_ERROR;
+		return (CC_ERROR);
 	cv_undo(el);
 	for (i = 0; i < el->el_state.argument; i++) {
 
 		c = *el->el_line.cursor;
-		if (iswupper(c))
-			*el->el_line.cursor = towlower(c);
-		else if (iswlower(c))
-			*el->el_line.cursor = towupper(c);
+		if (Isupper(c))
+			*el->el_line.cursor = Tolower(c);
+		else if (Islower(c))
+			*el->el_line.cursor = Toupper(c);
 
 		if (++el->el_line.cursor >= el->el_line.lastchar) {
 			el->el_line.cursor--;
@@ -282,16 +275,16 @@ vi_change_case(EditLine *el, wint_t c)
  *	Vi change prefix command
  *	[c]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_change_meta(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_change_meta(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	/*
          * Delete with insert == change: first we delete and then we leave in
          * insert mode.
          */
-	return cv_action(el, DELETE | INSERT);
+	return (cv_action(el, DELETE | INSERT));
 }
 
 
@@ -299,15 +292,15 @@ vi_change_meta(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi enter insert mode at the beginning of line
  *	[I]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_insert_at_bol(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_insert_at_bol(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	el->el_line.cursor = el->el_line.buffer;
 	cv_undo(el);
 	el->el_map.current = el->el_map.key;
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -315,9 +308,9 @@ vi_insert_at_bol(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi replace character under the cursor with the next character typed
  *	[r]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_replace_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_replace_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor >= el->el_line.lastchar)
@@ -326,7 +319,7 @@ vi_replace_char(EditLine *el, wint_t c __attribute__((__unused__)))
 	el->el_map.current = el->el_map.key;
 	el->el_state.inputmode = MODE_REPLACE_1;
 	cv_undo(el);
-	return CC_ARGHACK;
+	return (CC_ARGHACK);
 }
 
 
@@ -334,15 +327,15 @@ vi_replace_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi enter replace mode
  *	[R]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_replace_mode(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_replace_mode(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	el->el_map.current = el->el_map.key;
 	el->el_state.inputmode = MODE_REPLACE;
 	cv_undo(el);
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -350,14 +343,14 @@ vi_replace_mode(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi replace character under the cursor and enter insert mode
  *	[s]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_substitute_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_substitute_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	c_delafter(el, el->el_state.argument);
 	el->el_map.current = el->el_map.key;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -365,9 +358,9 @@ vi_substitute_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi substitute entire line
  *	[S]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_substitute_line(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_substitute_line(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	cv_undo(el);
@@ -375,7 +368,7 @@ vi_substitute_line(EditLine *el, wint_t c __attribute__((__unused__)))
 	    (int)(el->el_line.lastchar - el->el_line.buffer));
 	(void) em_kill_line(el, 0);
 	el->el_map.current = el->el_map.key;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -383,9 +376,9 @@ vi_substitute_line(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi change to end of line
  *	[C]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_change_to_eol(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_change_to_eol(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	cv_undo(el);
@@ -393,7 +386,7 @@ vi_change_to_eol(EditLine *el, wint_t c __attribute__((__unused__)))
 	    (int)(el->el_line.lastchar - el->el_line.cursor));
 	(void) ed_kill_line(el, 0);
 	el->el_map.current = el->el_map.key;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -401,14 +394,14 @@ vi_change_to_eol(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi enter insert mode
  *	[i]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_insert(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_insert(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	el->el_map.current = el->el_map.key;
 	cv_undo(el);
-	return CC_NORM;
+	return (CC_NORM);
 }
 
 
@@ -416,9 +409,9 @@ vi_insert(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi enter insert mode after the cursor
  *	[a]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_add(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_add(EditLine *el, Int c __attribute__((__unused__)))
 {
 	int ret;
 
@@ -433,7 +426,7 @@ vi_add(EditLine *el, wint_t c __attribute__((__unused__)))
 
 	cv_undo(el);
 
-	return (el_action_t)ret;
+	return (ret);
 }
 
 
@@ -441,15 +434,15 @@ vi_add(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi enter insert mode at end of line
  *	[A]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_add_at_eol(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_add_at_eol(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	el->el_map.current = el->el_map.key;
 	el->el_line.cursor = el->el_line.lastchar;
 	cv_undo(el);
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -457,12 +450,12 @@ vi_add_at_eol(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi delete prefix command
  *	[d]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_delete_meta(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_delete_meta(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	return cv_action(el, DELETE);
+	return (cv_action(el, DELETE));
 }
 
 
@@ -470,13 +463,13 @@ vi_delete_meta(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the end of the current space delimited word
  *	[E]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_end_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_end_big_word(EditLine *el, Int c)
 {
 
 	if (el->el_line.cursor == el->el_line.lastchar)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = cv__endword(el->el_line.cursor,
 	    el->el_line.lastchar, el->el_state.argument, cv__isWord);
@@ -484,9 +477,9 @@ vi_end_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
 	if (el->el_chared.c_vcmd.action != NOP) {
 		el->el_line.cursor++;
 		cv_delfini(el);
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -494,13 +487,13 @@ vi_end_big_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the end of the current word
  *	[e]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_end_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_end_word(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor == el->el_line.lastchar)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	el->el_line.cursor = cv__endword(el->el_line.cursor,
 	    el->el_line.lastchar, el->el_state.argument, cv__isword);
@@ -508,9 +501,9 @@ vi_end_word(EditLine *el, wint_t c __attribute__((__unused__)))
 	if (el->el_chared.c_vcmd.action != NOP) {
 		el->el_line.cursor++;
 		cv_delfini(el);
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -518,9 +511,9 @@ vi_end_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi undo last change
  *	[u]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_undo(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_undo(EditLine *el, Int c __attribute__((__unused__)))
 {
 	c_undo_t un = el->el_chared.c_undo;
 
@@ -537,7 +530,7 @@ vi_undo(EditLine *el, wint_t c __attribute__((__unused__)))
 	el->el_line.cursor = un.buf + un.cursor;
 	el->el_line.lastchar = un.buf + un.len;
 
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -545,9 +538,9 @@ vi_undo(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi enter command mode (use alternative key bindings)
  *	[<ESC>]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_command_mode(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_command_mode(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	/* [Esc] cancels pending action */
@@ -562,7 +555,7 @@ vi_command_mode(EditLine *el, wint_t c __attribute__((__unused__)))
 	if (el->el_line.cursor > el->el_line.buffer)
 		el->el_line.cursor--;
 #endif
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
@@ -570,8 +563,8 @@ vi_command_mode(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the beginning of line
  *	[0]
  */
-libedit_private el_action_t
-vi_zero(EditLine *el, wint_t c)
+protected el_action_t
+vi_zero(EditLine *el, Int c)
 {
 
 	if (el->el_state.doingarg)
@@ -580,27 +573,27 @@ vi_zero(EditLine *el, wint_t c)
 	el->el_line.cursor = el->el_line.buffer;
 	if (el->el_chared.c_vcmd.action != NOP) {
 		cv_delfini(el);
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 
 /* vi_delete_prev_char():
- *	Vi move to previous character (backspace)
+ * 	Vi move to previous character (backspace)
  *	[^H] in insert mode only
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_delete_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_delete_prev_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_line.cursor <= el->el_line.buffer)
-		return CC_ERROR;
+		return (CC_ERROR);
 
 	c_delbefore1(el);
 	el->el_line.cursor--;
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -608,34 +601,34 @@ vi_delete_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi list choices for completion or indicate end of file if empty line
  *	[^D]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_list_or_eof(EditLine *el, wint_t c)
+vi_list_or_eof(EditLine *el, Int c)
 {
 
 	if (el->el_line.cursor == el->el_line.lastchar) {
 		if (el->el_line.cursor == el->el_line.buffer) {
-			terminal_writec(el, c);	/* then do a EOF */
-			return CC_EOF;
+			term_writec(el, c);	/* then do a EOF */
+			return (CC_EOF);
 		} else {
 			/*
 			 * Here we could list completions, but it is an
 			 * error right now
 			 */
-			terminal_beep(el);
-			return CC_ERROR;
+			term_beep(el);
+			return (CC_ERROR);
 		}
 	} else {
 #ifdef notyet
 		re_goto_bottom(el);
 		*el->el_line.lastchar = '\0';	/* just in case */
-		return CC_LIST_CHOICES;
+		return (CC_LIST_CHOICES);
 #else
 		/*
 		 * Just complain for now.
 		 */
-		terminal_beep(el);
-		return CC_ERROR;
+		term_beep(el);
+		return (CC_ERROR);
 #endif
 	}
 }
@@ -645,11 +638,11 @@ vi_list_or_eof(EditLine *el, wint_t c)
  *	Vi cut from beginning of line to cursor
  *	[^U]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_kill_line_prev(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_kill_line_prev(EditLine *el, Int c __attribute__((__unused__)))
 {
-	wchar_t *kp, *cp;
+	Char *kp, *cp;
 
 	cp = el->el_line.buffer;
 	kp = el->el_chared.c_kill.buf;
@@ -658,7 +651,7 @@ vi_kill_line_prev(EditLine *el, wint_t c __attribute__((__unused__)))
 	el->el_chared.c_kill.last = kp;
 	c_delbefore(el, (int)(el->el_line.cursor - el->el_line.buffer));
 	el->el_line.cursor = el->el_line.buffer;	/* zap! */
-	return CC_REFRESH;
+	return (CC_REFRESH);
 }
 
 
@@ -666,12 +659,12 @@ vi_kill_line_prev(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi search history previous
  *	[?]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_search_prev(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_search_prev(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	return cv_search(el, ED_SEARCH_PREV_HISTORY);
+	return (cv_search(el, ED_SEARCH_PREV_HISTORY));
 }
 
 
@@ -679,12 +672,12 @@ vi_search_prev(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi search history next
  *	[/]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_search_next(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_search_next(EditLine *el, Int c __attribute__((__unused__)))
 {
 
-	return cv_search(el, ED_SEARCH_NEXT_HISTORY);
+	return (cv_search(el, ED_SEARCH_NEXT_HISTORY));
 }
 
 
@@ -692,15 +685,15 @@ vi_search_next(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi repeat current search in the same search direction
  *	[n]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_repeat_search_next(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_repeat_search_next(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_search.patlen == 0)
-		return CC_ERROR;
+		return (CC_ERROR);
 	else
-		return cv_repeat_srch(el, el->el_search.patdir);
+		return (cv_repeat_srch(el, el->el_search.patdir));
 }
 
 
@@ -709,12 +702,12 @@ vi_repeat_search_next(EditLine *el, wint_t c __attribute__((__unused__)))
  *	[N]
  */
 /*ARGSUSED*/
-libedit_private el_action_t
-vi_repeat_search_prev(EditLine *el, wint_t c __attribute__((__unused__)))
+protected el_action_t
+vi_repeat_search_prev(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	if (el->el_search.patlen == 0)
-		return CC_ERROR;
+		return (CC_ERROR);
 	else
 		return (cv_repeat_srch(el,
 		    el->el_search.patdir == ED_SEARCH_PREV_HISTORY ?
@@ -726,9 +719,9 @@ vi_repeat_search_prev(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the character specified next
  *	[f]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_next_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_next_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 	return cv_csearch(el, CHAR_FWD, -1, el->el_state.argument, 0);
 }
@@ -738,9 +731,9 @@ vi_next_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move to the character specified previous
  *	[F]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_prev_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 	return cv_csearch(el, CHAR_BACK, -1, el->el_state.argument, 0);
 }
@@ -750,9 +743,9 @@ vi_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move up to the character specified next
  *	[t]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_to_next_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_to_next_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 	return cv_csearch(el, CHAR_FWD, -1, el->el_state.argument, 1);
 }
@@ -762,9 +755,9 @@ vi_to_next_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi move up to the character specified previous
  *	[T]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_to_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_to_prev_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 	return cv_csearch(el, CHAR_BACK, -1, el->el_state.argument, 1);
 }
@@ -774,9 +767,9 @@ vi_to_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi repeat current character search in the same search direction
  *	[;]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_repeat_next_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_repeat_next_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 
 	return cv_csearch(el, el->el_search.chadir, el->el_search.chacha,
@@ -788,9 +781,9 @@ vi_repeat_next_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi repeat current character search in the opposite search direction
  *	[,]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_repeat_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_repeat_prev_char(EditLine *el, Int c __attribute__((__unused__)))
 {
 	el_action_t r;
 	int dir = el->el_search.chadir;
@@ -806,22 +799,22 @@ vi_repeat_prev_char(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi go to matching () {} or []
  *	[%]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_match(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_match(EditLine *el, Int c)
 {
-	const wchar_t match_chars[] = L"()[]{}";
-	wchar_t *cp;
+	const Char match_chars[] = STR("()[]{}");
+	Char *cp;
 	size_t delta, i, count;
-	wchar_t o_ch, c_ch;
+	Char o_ch, c_ch;
 
 	*el->el_line.lastchar = '\0';		/* just in case */
 
-	i = wcscspn(el->el_line.cursor, match_chars);
+	i = Strcspn(el->el_line.cursor, match_chars);
 	o_ch = el->el_line.cursor[i];
 	if (o_ch == 0)
 		return CC_ERROR;
-	delta = (size_t)(wcschr(match_chars, o_ch) - match_chars);
+	delta = Strchr(match_chars, o_ch) - match_chars;
 	c_ch = match_chars[delta ^ 1];
 	count = 1;
 	delta = 1 - (delta & 1) * 2;
@@ -844,18 +837,18 @@ vi_match(EditLine *el, wint_t c __attribute__((__unused__)))
 		if (delta > 0)
 			el->el_line.cursor++;
 		cv_delfini(el);
-		return CC_REFRESH;
+		return (CC_REFRESH);
 	}
-	return CC_CURSOR;
+	return (CC_CURSOR);
 }
 
 /* vi_undo_line():
  *	Vi undo all changes to line
  *	[U]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_undo_line(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_undo_line(EditLine *el, Int c)
 {
 
 	cv_undo(el);
@@ -867,9 +860,9 @@ vi_undo_line(EditLine *el, wint_t c __attribute__((__unused__)))
  *	[|]
  * NB netbsd vi goes to screen column 'n', posix says nth character
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_to_column(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_to_column(EditLine *el, Int c)
 {
 
 	el->el_line.cursor = el->el_line.buffer;
@@ -881,9 +874,9 @@ vi_to_column(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi yank to end of line
  *	[Y]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_yank_end(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_yank_end(EditLine *el, Int c)
 {
 
 	cv_yank(el, el->el_line.cursor,
@@ -895,9 +888,9 @@ vi_yank_end(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi yank
  *	[y]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_yank(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_yank(EditLine *el, Int c)
 {
 
 	return cv_action(el, YANK);
@@ -907,9 +900,9 @@ vi_yank(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi comment out current command
  *	[#]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_comment_out(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_comment_out(EditLine *el, Int c)
 {
 
 	el->el_line.cursor = el->el_line.buffer;
@@ -925,42 +918,50 @@ vi_comment_out(EditLine *el, wint_t c __attribute__((__unused__)))
  * NB: posix implies that we should enter insert mode, however
  * this is against historical precedent...
  */
-libedit_private el_action_t
+#ifdef __weak_reference
+__weakref_visible char *my_get_alias_text(const char *)
+    __weak_reference(get_alias_text);
+#endif
+protected el_action_t
 /*ARGSUSED*/
-vi_alias(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_alias(EditLine *el, Int c)
 {
+#ifdef __weak_reference
 	char alias_name[3];
-	const char *alias_text;
+	char *alias_text;
 
-	if (el->el_chared.c_aliasfun == NULL)
+	if (my_get_alias_text == 0) {
 		return CC_ERROR;
+	}
 
 	alias_name[0] = '_';
 	alias_name[2] = 0;
 	if (el_getc(el, &alias_name[1]) != 1)
 		return CC_ERROR;
 
-	alias_text = (*el->el_chared.c_aliasfun)(el->el_chared.c_aliasarg,
-	    alias_name);
+	alias_text = my_get_alias_text(alias_name);
 	if (alias_text != NULL)
-		el_wpush(el, ct_decode_string(alias_text, &el->el_scratch));
+		FUN(el,push)(el, ct_decode_string(alias_text, &el->el_scratch));
 	return CC_NORM;
+#else
+	return CC_ERROR;
+#endif
 }
 
 /* vi_to_history_line():
  *	Vi go to specified history file line.
  *	[G]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_to_history_line(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_to_history_line(EditLine *el, Int c)
 {
 	int sv_event_no = el->el_history.eventno;
 	el_action_t rval;
 
 
 	if (el->el_history.eventno == 0) {
-		 (void) wcsncpy(el->el_history.buf, el->el_line.buffer,
+		 (void) Strncpy(el->el_history.buf, el->el_line.buffer,
 		     EL_BUFSIZ);
 		 el->el_history.last = el->el_history.buf +
 			 (el->el_line.lastchar - el->el_line.buffer);
@@ -979,7 +980,7 @@ vi_to_history_line(EditLine *el, wint_t c __attribute__((__unused__)))
 		el->el_history.eventno = 1;
 		if (hist_get(el) == CC_ERROR)
 			return CC_ERROR;
-		el->el_history.eventno = 1 + el->el_history.ev.num
+		el->el_history.eventno = 1 + el->el_history.ev.num 
 					- el->el_state.argument;
 		if (el->el_history.eventno < 0) {
 			el->el_history.eventno = sv_event_no;
@@ -996,64 +997,70 @@ vi_to_history_line(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi edit history line with vi
  *	[v]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_histedit(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_histedit(EditLine *el, Int c)
 {
 	int fd;
 	pid_t pid;
 	ssize_t st;
 	int status;
 	char tempfile[] = "/tmp/histedit.XXXXXXXXXX";
-	char *cp = NULL;
+	char *cp;
 	size_t len;
-	wchar_t *line = NULL;
-	const char *editor;
+	Char *line;
 
 	if (el->el_state.doingarg) {
 		if (vi_to_history_line(el, 0) == CC_ERROR)
 			return CC_ERROR;
 	}
 
-	if ((editor = getenv("EDITOR")) == NULL)
-		editor = "vi";
 	fd = mkstemp(tempfile);
 	if (fd < 0)
 		return CC_ERROR;
 	len = (size_t)(el->el_line.lastchar - el->el_line.buffer);
 #define TMP_BUFSIZ (EL_BUFSIZ * MB_LEN_MAX)
-	cp = el_calloc(TMP_BUFSIZ, sizeof(*cp));
-	if (cp == NULL)
-		goto error;
-	line = el_calloc(len + 1, sizeof(*line));
-	if (line == NULL)
-		goto error;
-	wcsncpy(line, el->el_line.buffer, len);
+	cp = el_malloc(TMP_BUFSIZ);
+	if (cp == NULL) {
+		unlink(tempfile);
+		close(fd);
+		return CC_ERROR;
+	}
+	line = el_malloc(len * sizeof(*line));
+	if (line == NULL) {
+		el_free((ptr_t)cp);
+		return CC_ERROR;
+	}
+	Strncpy(line, el->el_line.buffer, len);
 	line[len] = '\0';
-	wcstombs(cp, line, TMP_BUFSIZ - 1);
+	ct_wcstombs(cp, line, TMP_BUFSIZ - 1);
 	cp[TMP_BUFSIZ - 1] = '\0';
 	len = strlen(cp);
 	write(fd, cp, len);
-	write(fd, "\n", (size_t)1);
+	write(fd, "\n", 1);
 	pid = fork();
 	switch (pid) {
 	case -1:
-		goto error;
+		close(fd);
+		unlink(tempfile);
+		el_free(cp);
+                el_free(line);
+		return CC_ERROR;
 	case 0:
 		close(fd);
-		execlp(editor, editor, tempfile, (char *)NULL);
+		execlp("vi", "vi", tempfile, (char *)NULL);
 		exit(0);
 		/*NOTREACHED*/
 	default:
 		while (waitpid(pid, &status, 0) != pid)
 			continue;
 		lseek(fd, (off_t)0, SEEK_SET);
-		st = read(fd, cp, TMP_BUFSIZ - 1);
+		st = read(fd, cp, TMP_BUFSIZ);
 		if (st > 0) {
-			cp[st] = '\0';
-			len = (size_t)(el->el_line.limit - el->el_line.buffer);
-			len = mbstowcs(el->el_line.buffer, cp, len);
-			if (len > 0 && el->el_line.buffer[len - 1] == '\n')
+			len = (size_t)(el->el_line.lastchar -
+			    el->el_line.buffer);
+			len = ct_mbstowcs(el->el_line.buffer, cp, len);
+			if (len > 0 && el->el_line.buffer[len -1] == '\n')
 				--len;
 		}
 		else
@@ -1069,12 +1076,6 @@ vi_histedit(EditLine *el, wint_t c __attribute__((__unused__)))
 	unlink(tempfile);
 	/* return CC_REFRESH; */
 	return ed_newline(el, 0);
-error:
-	el_free(line);
-	el_free(cp);
-	close(fd);
-	unlink(tempfile);
-	return CC_ERROR;
 }
 
 /* vi_history_word():
@@ -1083,33 +1084,33 @@ error:
  * Who knows where this one came from!
  * '_' in vi means 'entire current line', so 'cc' is a synonym for 'c_'
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_history_word(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_history_word(EditLine *el, Int c)
 {
-	const wchar_t *wp = HIST_FIRST(el);
-	const wchar_t *wep, *wsp;
+	const Char *wp = HIST_FIRST(el);
+	const Char *wep, *wsp;
 	int len;
-	wchar_t *cp;
-	const wchar_t *lim;
+	Char *cp;
+	const Char *lim;
 
 	if (wp == NULL)
 		return CC_ERROR;
 
-	wep = wsp = NULL;
+	wep = wsp = 0;
 	do {
-		while (iswspace(*wp))
+		while (Isspace(*wp))
 			wp++;
 		if (*wp == 0)
 			break;
 		wsp = wp;
-		while (*wp && !iswspace(*wp))
+		while (*wp && !Isspace(*wp))
 			wp++;
 		wep = wp;
 	} while ((!el->el_state.doingarg || --el->el_state.argument > 0)
 	    && *wp != 0);
 
-	if (wsp == NULL || (el->el_state.doingarg && el->el_state.argument != 0))
+	if (wsp == 0 || (el->el_state.doingarg && el->el_state.argument != 0))
 		return CC_ERROR;
 
 	cv_undo(el);
@@ -1133,9 +1134,9 @@ vi_history_word(EditLine *el, wint_t c __attribute__((__unused__)))
  *	Vi redo last non-motion command
  *	[.]
  */
-libedit_private el_action_t
+protected el_action_t
 /*ARGSUSED*/
-vi_redo(EditLine *el, wint_t c __attribute__((__unused__)))
+vi_redo(EditLine *el, Int c)
 {
 	c_redo_t *r = &el->el_chared.c_redo;
 
@@ -1151,10 +1152,10 @@ vi_redo(EditLine *el, wint_t c __attribute__((__unused__)))
 			/* sanity */
 			r->pos = r->lim - 1;
 		r->pos[0] = 0;
-		el_wpush(el, r->buf);
+		FUN(el,push)(el, r->buf);
 	}
 
 	el->el_state.thiscmd = r->cmd;
 	el->el_state.thisch = r->ch;
-	return (*el->el_map.func[r->cmd])(el, r->ch);
+	return  (*el->el_map.func[r->cmd])(el, r->ch);
 }
